@@ -1,3 +1,16 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+
+VERSION_EXPLANATION = (
+    'You might be seeing this error because you\'re using the wrong Compose '
+    'file version. Either specify a version of "2" (or "2.0") and place your '
+    'service definitions under the `services` key, or omit the `version` key '
+    'and place your service definitions at the root of the file to use '
+    'version 1.\nFor more on the Compose file format versions, see '
+    'https://docs.docker.com/compose/compose-file/')
+
+
 class ConfigurationError(Exception):
     def __init__(self, msg):
         self.msg = msg
@@ -26,7 +39,8 @@ class CircularReference(ConfigurationError):
 class ComposeFileNotFound(ConfigurationError):
     def __init__(self, supported_filenames):
         super(ComposeFileNotFound, self).__init__("""
-        Can't find a suitable configuration file in this directory or any parent. Are you in the right directory?
+        Can't find a suitable configuration file in this directory or any
+        parent. Are you in the right directory?
 
         Supported filenames: %s
         """ % ", ".join(supported_filenames))
