@@ -2,11 +2,13 @@
 Integration tests which cover state convergence (aka smart recreate) performed
 by `docker-compose up`.
 """
+from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import py
 
 from .testcases import DockerClientTestCase
+from .testcases import get_links
 from compose.config import config
 from compose.project import Project
 from compose.service import ConvergenceStrategy
@@ -25,10 +27,10 @@ class ProjectTestCase(DockerClientTestCase):
         details = config.ConfigDetails(
             'working_dir',
             [config.ConfigFile(None, cfg)])
-        return Project.from_dicts(
+        return Project.from_config(
             name='composetest',
             client=self.client,
-            service_dicts=config.load(details))
+            config_data=config.load(details))
 
 
 class BasicProjectTest(ProjectTestCase):
@@ -36,8 +38,8 @@ class BasicProjectTest(ProjectTestCase):
         super(BasicProjectTest, self).setUp()
 
         self.cfg = {
-            'db': {'image': 'busybox:latest'},
-            'web': {'image': 'busybox:latest'},
+            'db': {'image': 'busybox:latest', 'command': 'top'},
+            'web': {'image': 'busybox:latest', 'command': 'top'},
         }
 
     def test_no_change(self):
@@ -186,8 +188,8 @@ class ProjectWithDependenciesTest(ProjectTestCase):
         web, = [c for c in containers if c.service == 'web']
         nginx, = [c for c in containers if c.service == 'nginx']
 
-        self.assertEqual(web.links(), ['composetest_db_1', 'db', 'db_1'])
-        self.assertEqual(nginx.links(), ['composetest_web_1', 'web', 'web_1'])
+        self.assertEqual(set(get_links(web)), {'composetest_db_1', 'db', 'db_1'})
+        self.assertEqual(set(get_links(nginx)), {'composetest_web_1', 'web', 'web_1'})
 
 
 class ServiceStateTest(DockerClientTestCase):
@@ -264,13 +266,13 @@ class ServiceStateTest(DockerClientTestCase):
         dockerfile = context.join('Dockerfile')
         dockerfile.write(base_image)
 
-        web = self.create_service('web', build=str(context))
+        web = self.create_service('web', build={'context': str(context)})
         container = web.create_container()
 
         dockerfile.write(base_image + 'CMD echo hello world\n')
         web.build()
 
-        web = self.create_service('web', build=str(context))
+        web = self.create_service('web', build={'context': str(context)})
         self.assertEqual(('recreate', [container]), web.convergence_plan())
 
     def test_image_changed_to_build(self):
@@ -284,7 +286,7 @@ class ServiceStateTest(DockerClientTestCase):
         web = self.create_service('web', image='busybox')
         container = web.create_container()
 
-        web = self.create_service('web', build=str(context))
+        web = self.create_service('web', build={'context': str(context)})
         plan = web.convergence_plan()
         self.assertEqual(('recreate', [container]), plan)
         containers = web.execute_convergence_plan(plan)
