@@ -3,6 +3,9 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import os
+import shutil
+import tempfile
+from io import StringIO
 
 import docker
 import py
@@ -10,13 +13,14 @@ import pytest
 
 from .. import mock
 from .. import unittest
+from ..helpers import build_config
 from compose.cli.command import get_project
 from compose.cli.command import get_project_name
 from compose.cli.docopt_command import NoSuchCommand
 from compose.cli.errors import UserError
 from compose.cli.main import TopLevelCommand
 from compose.const import IS_WINDOWS_PLATFORM
-from compose.service import Service
+from compose.project import Project
 
 
 class CLITestCase(unittest.TestCase):
@@ -42,20 +46,36 @@ class CLITestCase(unittest.TestCase):
         project_name = get_project_name(None, project_name=name)
         self.assertEquals('explicitprojectname', project_name)
 
-    def test_project_name_from_environment_old_var(self):
-        name = 'namefromenv'
-        with mock.patch.dict(os.environ):
-            os.environ['FIG_PROJECT_NAME'] = name
-            project_name = get_project_name(None)
-        self.assertEquals(project_name, name)
-
+    @mock.patch.dict(os.environ)
     def test_project_name_from_environment_new_var(self):
         name = 'namefromenv'
-        with mock.patch.dict(os.environ):
-            os.environ['COMPOSE_PROJECT_NAME'] = name
-            project_name = get_project_name(None)
+        os.environ['COMPOSE_PROJECT_NAME'] = name
+        project_name = get_project_name(None)
         self.assertEquals(project_name, name)
 
+    def test_project_name_with_empty_environment_var(self):
+        base_dir = 'tests/fixtures/simple-composefile'
+        with mock.patch.dict(os.environ):
+            os.environ['COMPOSE_PROJECT_NAME'] = ''
+            project_name = get_project_name(base_dir)
+        self.assertEquals('simplecomposefile', project_name)
+
+    @mock.patch.dict(os.environ)
+    def test_project_name_with_environment_file(self):
+        base_dir = tempfile.mkdtemp()
+        try:
+            name = 'namefromenvfile'
+            with open(os.path.join(base_dir, '.env'), 'w') as f:
+                f.write('COMPOSE_PROJECT_NAME={}'.format(name))
+            project_name = get_project_name(base_dir)
+            assert project_name == name
+
+            # Environment has priority over .env file
+            os.environ['COMPOSE_PROJECT_NAME'] = 'namefromenv'
+            assert get_project_name(base_dir) == os.environ['COMPOSE_PROJECT_NAME']
+        finally:
+            shutil.rmtree(base_dir)
+
     def test_get_project(self):
         base_dir = 'tests/fixtures/longer-filename-composefile'
         project = get_project(base_dir)
@@ -63,76 +83,71 @@ class CLITestCase(unittest.TestCase):
         self.assertTrue(project.client)
         self.assertTrue(project.services)
 
-    def test_help(self):
-        command = TopLevelCommand()
-        with self.assertRaises(SystemExit):
-            command.dispatch(['-h'], None)
-
     def test_command_help(self):
-        with self.assertRaises(SystemExit) as ctx:
-            TopLevelCommand().dispatch(['help', 'up'], None)
+        with mock.patch('sys.stdout', new=StringIO()) as fake_stdout:
+            TopLevelCommand.help({'COMMAND': 'up'})
 
-        self.assertIn('Usage: up', str(ctx.exception))
-
-    def test_command_help_dashes(self):
-        with self.assertRaises(SystemExit) as ctx:
-            TopLevelCommand().dispatch(['help', 'migrate-to-labels'], None)
-
-        self.assertIn('Usage: migrate-to-labels', str(ctx.exception))
+        assert "Usage: up" in fake_stdout.getvalue()
 
     def test_command_help_nonexistent(self):
-        with self.assertRaises(NoSuchCommand):
-            TopLevelCommand().dispatch(['help', 'nonexistent'], None)
+        with pytest.raises(NoSuchCommand):
+            TopLevelCommand.help({'COMMAND': 'nonexistent'})
 
     @pytest.mark.xfail(IS_WINDOWS_PLATFORM, reason="requires dockerpty")
-    @mock.patch('compose.cli.main.dockerpty', autospec=True)
-    def test_run_with_environment_merged_with_options_list(self, mock_dockerpty):
-        command = TopLevelCommand()
+    @mock.patch('compose.cli.main.RunOperation', autospec=True)
+    @mock.patch('compose.cli.main.PseudoTerminal', autospec=True)
+    def test_run_interactive_passes_logs_false(self, mock_pseudo_terminal, mock_run_operation):
         mock_client = mock.create_autospec(docker.Client)
-        mock_project = mock.Mock(client=mock_client)
-        mock_project.get_service.return_value = Service(
-            'service',
+        project = Project.from_config(
+            name='composetest',
             client=mock_client,
-            environment=['FOO=ONE', 'BAR=TWO'],
-            image='someimage')
+            config_data=build_config({
+                'service': {'image': 'busybox'}
+            }),
+        )
+        command = TopLevelCommand(project)
 
-        command.run(mock_project, {
-            'SERVICE': 'service',
-            'COMMAND': None,
-            '-e': ['BAR=NEW', 'OTHER=bär'.encode('utf-8')],
-            '--user': None,
-            '--no-deps': None,
-            '--allow-insecure-ssl': None,
-            '-d': True,
-            '-T': None,
-            '--entrypoint': None,
-            '--service-ports': None,
-            '--publish': [],
-            '--rm': None,
-            '--name': None,
-        })
+        with pytest.raises(SystemExit):
+            command.run({
+                'SERVICE': 'service',
+                'COMMAND': None,
+                '-e': [],
+                '--user': None,
+                '--no-deps': None,
+                '-d': False,
+                '-T': None,
+                '--entrypoint': None,
+                '--service-ports': None,
+                '--publish': [],
+                '--rm': None,
+                '--name': None,
+                '--workdir': None,
+            })
 
-        _, _, call_kwargs = mock_client.create_container.mock_calls[0]
-        self.assertEqual(
-            call_kwargs['environment'],
-            {'FOO': 'ONE', 'BAR': 'NEW', 'OTHER': u'bär'})
+        _, _, call_kwargs = mock_run_operation.mock_calls[0]
+        assert call_kwargs['logs'] is False
 
     def test_run_service_with_restart_always(self):
-        command = TopLevelCommand()
         mock_client = mock.create_autospec(docker.Client)
-        mock_project = mock.Mock(client=mock_client)
-        mock_project.get_service.return_value = Service(
-            'service',
+
+        project = Project.from_config(
+            name='composetest',
             client=mock_client,
-            restart={'Name': 'always', 'MaximumRetryCount': 0},
-            image='someimage')
-        command.run(mock_project, {
+            config_data=build_config({
+                'service': {
+                    'image': 'busybox',
+                    'restart': 'always',
+                }
+            }),
+        )
+
+        command = TopLevelCommand(project)
+        command.run({
             'SERVICE': 'service',
             'COMMAND': None,
             '-e': [],
             '--user': None,
             '--no-deps': None,
-            '--allow-insecure-ssl': None,
             '-d': True,
             '-T': None,
             '--entrypoint': None,
@@ -140,6 +155,7 @@ class CLITestCase(unittest.TestCase):
             '--publish': [],
             '--rm': None,
             '--name': None,
+            '--workdir': None,
         })
 
         self.assertEquals(
@@ -147,21 +163,13 @@ class CLITestCase(unittest.TestCase):
             'always'
         )
 
-        command = TopLevelCommand()
-        mock_client = mock.create_autospec(docker.Client)
-        mock_project = mock.Mock(client=mock_client)
-        mock_project.get_service.return_value = Service(
-            'service',
-            client=mock_client,
-            restart='always',
-            image='someimage')
-        command.run(mock_project, {
+        command = TopLevelCommand(project)
+        command.run({
             'SERVICE': 'service',
             'COMMAND': None,
             '-e': [],
             '--user': None,
             '--no-deps': None,
-            '--allow-insecure-ssl': None,
             '-d': True,
             '-T': None,
             '--entrypoint': None,
@@ -169,6 +177,7 @@ class CLITestCase(unittest.TestCase):
             '--publish': [],
             '--rm': True,
             '--name': None,
+            '--workdir': None,
         })
 
         self.assertFalse(
@@ -176,24 +185,22 @@ class CLITestCase(unittest.TestCase):
         )
 
     def test_command_manula_and_service_ports_together(self):
-        command = TopLevelCommand()
-        mock_client = mock.create_autospec(docker.Client)
-        mock_project = mock.Mock(client=mock_client)
-        mock_project.get_service.return_value = Service(
-            'service',
-            client=mock_client,
-            restart='always',
-            image='someimage',
+        project = Project.from_config(
+            name='composetest',
+            client=None,
+            config_data=build_config({
+                'service': {'image': 'busybox'},
+            }),
         )
+        command = TopLevelCommand(project)
 
         with self.assertRaises(UserError):
-            command.run(mock_project, {
+            command.run({
                 'SERVICE': 'service',
                 'COMMAND': None,
                 '-e': [],
                 '--user': None,
                 '--no-deps': None,
-                '--allow-insecure-ssl': None,
                 '-d': True,
                 '-T': None,
                 '--entrypoint': None,
