@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import docker
@@ -11,8 +12,9 @@ from compose.container import get_container_name
 class ContainerTest(unittest.TestCase):
 
     def setUp(self):
+        self.container_id = "abcabcabcbabc12345"
         self.container_dict = {
-            "Id": "abc",
+            "Id": self.container_id,
             "Image": "busybox:latest",
             "Command": "top",
             "Created": 1387384730,
@@ -40,19 +42,22 @@ class ContainerTest(unittest.TestCase):
         self.assertEqual(
             container.dictionary,
             {
-                "Id": "abc",
+                "Id": self.container_id,
                 "Image": "busybox:latest",
                 "Name": "/composetest_db_1",
             })
 
     def test_from_ps_prefixed(self):
-        self.container_dict['Names'] = ['/swarm-host-1' + n for n in self.container_dict['Names']]
+        self.container_dict['Names'] = [
+            '/swarm-host-1' + n for n in self.container_dict['Names']
+        ]
 
-        container = Container.from_ps(None,
-                                      self.container_dict,
-                                      has_been_inspected=True)
+        container = Container.from_ps(
+            None,
+            self.container_dict,
+            has_been_inspected=True)
         self.assertEqual(container.dictionary, {
-            "Id": "abc",
+            "Id": self.container_id,
             "Image": "busybox:latest",
             "Name": "/composetest_db_1",
         })
@@ -141,13 +146,19 @@ class ContainerTest(unittest.TestCase):
         self.assertEqual(container.get('HostConfig.VolumesFrom'), ["volume_id"])
         self.assertEqual(container.get('Foo.Bar.DoesNotExist'), None)
 
+    def test_short_id(self):
+        container = Container(None, self.container_dict, has_been_inspected=True)
+        assert container.short_id == self.container_id[:12]
+
 
 class GetContainerNameTestCase(unittest.TestCase):
 
     def test_get_container_name(self):
         self.assertIsNone(get_container_name({}))
         self.assertEqual(get_container_name({'Name': 'myproject_db_1'}), 'myproject_db_1')
-        self.assertEqual(get_container_name({'Names': ['/myproject_db_1', '/myproject_web_1/db']}), 'myproject_db_1')
+        self.assertEqual(
+            get_container_name({'Names': ['/myproject_db_1', '/myproject_web_1/db']}),
+            'myproject_db_1')
         self.assertEqual(
             get_container_name({
                 'Names': [
